import { Component, h } from 'preact';

/** Creates a new store, which is a tiny evented state container.
 *  @name createStore
 *  @param {Object} [state={}]		Optional initial state
 *  @returns {store}
 *  @example
 *    let store = createStore();
 *    store.subscribe( state => console.log(state) );
 *    store.setState({ a: 'b' });   // logs { a: 'b' }
 *    store.setState({ c: 'd' });   // logs { c: 'd' }
 */
function createStore(state) {
	var listeners = [];
	state = state || {};

	/** An observable state container, returned from {@link createStore}
	 *  @name store
	 */

	return /** @lends store */ {

		/** Apply a partial state object to the current state, invoking registered listeners.
		 *  @param {Object} update				An object with properties to be merged into state
		 *  @param {Boolean} [overwrite=false]	If `true`, update will replace state instead of being merged into it
		 */
		setState: function setState(update, overwrite) {
			state = overwrite ? update : assign(assign({}, state), update);
			for (var i=0; i<listeners.length; i++) { listeners[i](state); }
		},

		/** Register a listener function to be called whenever state is changed.
		 *  @param {Function} listener
		 */
		subscribe: function subscribe(listener) {
			listeners.push(listener);
		},

		/** Remove a previously-registered listener function.
		 *  @param {Function} listener
		 */
		unsubscribe: function unsubscribe(listener) {
			var i = listeners.indexOf(listener);
			listeners.splice(i, !!~i);
		},

		/** Retreive the current state object.
		 *  @returns {Object} state
		 */
		getState: function getState() {
			return state;
		}
	};
}


/** Wire a component up to the store. Passes state as props, re-renders on change.
 *  @param {Function|Array|String} mapStateToProps  A function mapping of store state to prop values, or an array/CSV of properties to map.
 *  @param {Function|Object} [actions] 				Action functions (pure state mappings), or a factory returning them.
 *  @returns {Component} ConnectedComponent
 *  @example
 *    const Foo = connect('foo,bar')( ({ foo, bar }) => <div /> )
 *  @example
 *    @connect( state => ({ foo: state.foo, bar: state.bar }) )
 *    export class Foo { render({ foo, bar }) { } }
 */
function connect(mapStateToProps, actions) {
	if (typeof mapStateToProps!=='function') {
		mapStateToProps = select(mapStateToProps || []);
	}
	return function (Child) {
		function Wrapper(props, ref) {
			var this$1 = this;
			var store = ref.store;

			var state = mapStateToProps(store ? store.getState() : {}, props);
			var boundActions = actions ? mapActions(actions, store) : { store: store };
			var update = function () {
				var mapped = mapStateToProps(store ? store.getState() : {}, this$1.props);
				if (!shallowEqual(mapped, state)) {
					state = mapped;
					this$1.setState(null);
				}
			};
			this.componentDidMount = function () {
				store.subscribe(update);
			};
			this.componentWillUnmount = function () {
				store.unsubscribe(update);
			};
			this.render = function (props) { return h(Child, assign(assign(assign({}, boundActions), props), state)); };
		}
		return (Wrapper.prototype = new Component()).constructor = Wrapper;
	};
}


/** Provider exposes a store (passed as `props.store`) into context.
 *
 *  Generally, an entire application is wrapped in a single `<Provider>` at the root.
 *  @class
 *  @extends Component
 *  @param {Object} props
 *  @param {Store} props.store		A {Store} instance to expose via context.
 */
function Provider(){}
Provider.prototype.getChildContext = function() {
	return { store: this.props.store };
};
Provider.prototype.render = function(props) {
	return props.children[0];
};


// Bind an object/factory of actions to the store and wrap them.
function mapActions(actions, store) {
	if (typeof actions==='function') { actions = actions(store); }
	var mapped = {};
	for (var i in actions) {
		mapped[i] = createAction(store, actions[i]);
	}
	return mapped;
}


// Bind a single action to the store and sequester its return value.
function createAction(store, action) {
	var arguments$1 = arguments;

	var args = [store.getState()];
	for (var i=0; i<arguments.length; i++) { args.push(arguments$1[i]); }
	var ret = action.apply(store, args);
	if (ret!=null) {
		if (ret.then) { ret.then(store.setState); }
		else { store.setState(ret); }
	}
}


// select('foo,bar') creates a function of the form: ({ foo, bar }) => ({ foo, bar })
function select(properties) {
	if (typeof properties==='string') { properties = properties.split(','); }
	return function (state) {
		var selected = {};
		for (var i=properties.length; i--; ) {
			selected[properties[i]] = state[properties[i]];
		}
		return selected;
	};
}


// Returns a boolean indicating if all keys and values match between two objects.
function shallowEqual(a, b) {
	for (var i in a) { if (a[i]!==b[i]) { return false; } }
	for (var i$1 in b) { if (!(i$1 in a)) { return false; } }
	return true;
}


// Lighter Object.assign stand-in
function assign(obj, props) {
	for (var i in props) { obj[i] = props[i]; }
	return obj;
}

export { createStore, connect, Provider };
//# sourceMappingURL=unistore.es.js.map
